--!parent
--!encoding=utf-8

--****************************************************************************
-- @file			LSW CORE LIB: DIRECTSHOW
-- @author		Valentin Schmidt
-- @version	 0.6
-- @requires	xtra("DirectShow")
--****************************************************************************

property ancestor

property $

property _handler
property _target

-- constants
property EVENT
property CLSID
property WINDOWSTYLE
property MEDIATYPE

----------------------------------------
-- @constructor
----------------------------------------
on new (me, props)
	
	-- win only
	if $.OS="mac" then return false

	-- libs
	$.import("detacher")

	$.loadXtras(me.libdir, ["DirectShow.x32"], [])
	--put xtra("DirectShow").interface()
	
	if voidP(props) then
		me.ancestor = xtra("DirectShow").new()
	else
		me.ancestor = xtra("DirectShow").new(props)
	end if

	-- load constants
	me.loadEventConstants()
	me.loadGuidConstants()
	me.loadWindowStyleConstants()
	me.loadMediaTypeConstants()
	
	return me
end

----------------------------------------
-- Adds callback for DirectShow events
-- @param {symbol} cbHandler
-- @param {object} [cbTarget=_movie]
----------------------------------------
on setCallback (me, cbHandler, cbTarget)
	if voidp(cbTarget) then cbTarget = _movie

	me._handler = cbHandler
	me._target = cbTarget

	props = [:]
	props["callback"] = #_eventcallback
	props["callbacktarget"] = me

	me.ds_set_param(props)
end

----------------------------------------
-- @callback
----------------------------------------
on _eventcallback (me, evt, param1, param2)	
	if symbolP(me._handler) then
		me.$.detacher.detach(me._handler, me._target, evt, param1, param2)
	end if
end

----------------------------------------
--
----------------------------------------
on loadEventConstants (me)

	const = [:]

	const[#EC_ACTIVATE]=19
	const[#EC_BUFFERING_DATA]=17
	const[#EC_BUILT]=768
	const[#EC_CLOCK_CHANGED]=13
	const[#EC_CLOCK_UNSET]=81
	const[#EC_CODECAPI_EVENT]=87
	const[#EC_COMPLETE]=1
	const[#EC_DEVICE_LOST]=31
	const[#EC_DISPLAY_CHANGED]=22
	const[#EC_END_OF_SEGMENT]=28
	const[#EC_ERRORABORT]=3
	const[#EC_ERROR_STILLPLAYING]=8
	const[#EC_EXTDEVICE_MODE_CHANGE]=49
	const[#EC_FULLSCREEN_LOST]=18
	const[#EC_GRAPH_CHANGED]=80
	const[#EC_LENGTH_CHANGED]=30
	const[#EC_NEED_RESTART]=20
	const[#EC_NOTIFY_WINDOW]=25
	const[#EC_OLE_EVENT]=24
	const[#EC_OPENING_FILE]=16
	const[#EC_PALETTE_CHANGED]=9
	const[#EC_PAUSED]=14
	const[#EC_PREPROCESS_COMPLETE]=86
	const[#EC_QUALITY_CHANGE]=11
	const[#EC_REPAINT]=5
	const[#EC_SEGMENT_STARTED]=29
	const[#EC_SHUTTING_DOWN]=12
	const[#EC_SNDDEV_IN_ERROR]=512
	const[#EC_SNDDEV_OUT_ERROR]=513
	const[#EC_STARVATION]=23
	const[#EC_STATE_CHANGE]=50
	const[#EC_STEP_COMPLETE]=36
	const[#EC_STREAM_CONTROL_STARTED]=27
	const[#EC_STREAM_CONTROL_STOPPED]=26
	const[#EC_STREAM_ERROR_STILLPLAYING]=7
	const[#EC_STREAM_ERROR_STOPPED]=6
	const[#EC_SYSTEMBASE]=0
	const[#EC_TIME]=4
	const[#EC_TIMECODE_AVAILABLE]=48
	const[#EC_UNBUILT]=769
	const[#EC_USER]=32768
	const[#EC_USERABORT]=2
	const[#EC_VIDEO_SIZE_CHANGED]=10
	const[#EC_VMR_RECONNECTION_FAILED]=85
	const[#EC_VMR_RENDERDEVICE_SET]=83
	const[#EC_VMR_SURFACE_FLIPPED]=84
	const[#EC_WINDOW_DESTROYED]=21
	const[#EC_WMT_EVENT]=594
	const[#EC_WMT_INDEX_EVENT]=593

	me.EVENT = const
end

----------------------------------------
--
----------------------------------------
on loadGuidConstants (me)

	const = [:]

	-- filter categories
	const[#LegacyAmFilterCategory]	 = "{083863F1-70DE-11d0-BD40-00A0C911CE86}"
	const[#VideoCompressorCategory]	= "{33D9A760-90C8-11D0-BD43-00A0C911CE86}"
	const[#AudioCompressorCategory]	= "{33D9A761-90C8-11D0-BD43-00A0C911CE86}"
	const[#VideoInputDeviceCategory] = "{860BB310-5D01-11D0-BD3B-00A0C911CE86}"
	const[#AudioInputDeviceCategory] = "{33D9A762-90C8-11D0-BD43-00A0C911CE86}"
	const[#AudioRendererCategory]		= "{E0F158E1-CB04-11D0-BD4E-00A0C911CE86}"
	const[#MidiRendererCategory]		 = "{4EFE2452-168A-11D1-BC76-00C04FB9453B}"
	const[#MediaEncoderCategory]		 = "{7D22E920-5CA9-4787-8C2B-A6779BD11781}"
	const[#MediaMultiplexerCategory] = "{236C9559-ADCE-4736-BF72-BAB34E392196}"

	-- DMO categories
	const[#DMOCATEGORY_AUDIO_CAPTURE_EFFECT] = "{F665AABA-3E09-4920-AA5F-219811148F09}"
	const[#DMOCATEGORY_AUDIO_DECODER]				= "{57F2DB8B-E6BB-4513-9D43-DCD2A6593125}"
	const[#DMOCATEGORY_AUDIO_EFFECT]				 = "{F3602B3F-0592-48DF-A4CD-674721E7EBEB}"
	const[#DMOCATEGORY_AUDIO_ENCODER]				= "{33D9A761-90C8-11D0-BD43-00A0C911CE86}"
	const[#DMOCATEGORY_AUDIO_NOISE_SUPPRESS] = "{E07F903F-62FD-4E60-8CDD-DEA7236665B5}"
	const[#DMOCATEGORY_VIDEO_DECODER]				= "{4A69B442-28BE-4991-969C-B500ADF5D8A8}"
	const[#DMOCATEGORY_VIDEO_EFFECT]				 = "{D990EE14-776C-4723-BE46-3DA2F56F10B9}"
	const[#DMOCATEGORY_VIDEO_ENCODER]				= "{33D9A760-90C8-11D0-BD43-00A0C911CE86}"

	-- LAV filters
	const[#LAVSplitter]			 = "{171252A0-8820-4AFE-9DF8-5C92B2D66B04}"
	const[#LAVSplitterSource] = "{B98D13E7-55DB-4385-A33D-09FD1BA26338}"
	const[#LAVVideoDecoder]	 = "{EE30215D-164F-4A92-A4EB-9D4C13390F9F}"
	const[#LAVAudioDecoder]	 = "{E8E73B6B-4CB3-44A4-BE99-4F7BCB96E491}"

	-- VSFilter/DirectVobSub/DirectVobSubXy
	const[#VSFilter] = "{93A22E7A-5091-45EF-BA61-6DA26156A5D0}"
	const[#VSFilter_autoload] = "{9852A670-F845-491B-9BE6-EBD841B8A613}"

	const[#WavDest]					= "{3C78B8E2-6C4D-11D1-ADE2-0000F8754B99}"
	const[#FileWriter]			 = "{8596E5F0-0DA5-11D0-BD21-00A0C911CE86}"

	-- DS Xtra filters
	const[#DSXtraDumpFilter]			 = "36a5f770-fe4c-11ce-a8ed-00aa002feab5"
	const[#DSXtraAudioSourceFilter]		 = "BFBC37B0-20B1-43CE-B7AB-6DCD080D1E46"
	const[#DSXtraVideoSourceFilter]		 = "9A80E195-3BBA-4821-B18B-21BB496F80F8"

	-- Video renderers
	const[#VideoRenderer] = "{70E102B0-5556-11CE-97C0-00AA0055595A}"
	const[#VideoMixingRenderer] = "{B87BEB7B-8D29-423f-AE4D-6582C10175AC}"
	const[#VideoRendererDefault] = "{6BC1CFFA-8FC1-4261-AC22-CFB4CC38DB50}"

	-- CLSID_VideoRendererDefault = alias for CLSID_VideoMixingRenderer on systems
	-- that support VMR7 (XP or newer) or CLSID_VideoRenderer otherwise (but on those
	-- systems I think that CLSID_VideoRenderer is not registered).

	const[#VideoMixingRenderer9] = "{51B4ABF3-748F-4E3B-A276-C828330E926A}"
	const[#EnhancedVideoRenderer] = "{FA10746C-9B63-4B6C-BC49-FC300EA5F256}"

	-- Audio renderes
	const[#DefaultDirectSoundDevice] = "{79376820-07D0-11CF-A24D-0020AFD79767}"
	const[#DefaultWaveOutDevice] = "{E30629D1-27E5-11CE-875D-00608CB78066}"
	
	-- MIDI renderers
	const[#MIDIRenderer] = "{07B65360-C445-11CE-AFDE-00AA006C14F4}"

	-- other MS filters
	const[#MicrosoftDTVDVDAudioDecoder] = "{E1F1A0B8-BEEE-490D-BA7C-066C40B5E2B9}"
	const[#MicrosoftDTVDVDVideoDecoder] = "{212690FB-83E5-4526-8FD7-74478B7939CD}"
	const[#OverlayMixer] = "{CD8743A1-3736-11D0-9E69-00C04FD7C15B}"
	const[#OverlayMixer2] = "{A0025E90-E45B-11D1-ABE9-00A0C905F375}"
	const[#FileSourceAsync] = "{E436EBB5-524F-11CE-9F53-0020AF0BA770}"
	const[#FileSourceURL] = "{E436EBB6-524F-11CE-9F53-0020AF0BA770}"
	const[#WaveParser] = "{D51BD5A1-7548-11CF-A520-0080C77EF58A}"
	const[#AVISplitter] = "{1B544C20-FD0B-11CE-8C63-00AA0044B51E}"
	const[#MIDIParser] = "{D51BD5A2-7548-11CF-A520-0080C77EF58A}"
	const[#AVIWAVFileSource] = "{D3588AB0-0781-11CE-B03A-0020AF0BA770}"
	const[#ColorSpaceConverter] = "{1643E180-90F5-11CE-97D5-00AA0055595A}"
	const[#AudioRecorderWAVDest] = "{E882F102-F626-49E9-BD68-CE2BE7E59EA0}"
	const[#AudioRecorderWaveForm] = "{E882F102-F626-49E9-BD68-CE2BE7E59EB0}"

	-- new
	const[#WindowsMediaSourceFilter] = "{6B6D0800-9ADA-11D0-A520-00A0D10129C0}"
	
	me.CLSID = const
end

----------------------------------------
-- Window Styles
-- https://msdn.microsoft.com/de-de/library/windows/desktop/ms632600(v=vs.85).aspx
----------------------------------------
on loadWindowStyleConstants (me)

	const = [:]

	const[#WS_BORDER] = 8388608
	const[#WS_CAPTION] = 12582912
	const[#WS_CHILD] = 1073741824
	const[#WS_CHILDWINDOW] = 1073741824
	const[#WS_CLIPCHILDREN] = 33554432
	const[#WS_CLIPSIBLINGS] = 67108864
	const[#WS_DISABLED] = 134217728
	const[#WS_DLGFRAME] = 4194304
	const[#WS_GROUP] = 131072
	const[#WS_HSCROLL] = 1048576
	const[#WS_ICONIC] = 536870912
	const[#WS_MAXIMIZE] = 16777216
	const[#WS_MAXIMIZEBOX] = 65536
	const[#WS_MINIMIZE] = 536870912
	const[#WS_MINIMIZEBOX] = 131072
	const[#WS_OVERLAPPED] = 0
	const[#WS_POPUP] = 2147483648
	const[#WS_SIZEBOX] = 262144
	const[#WS_SYSMENU] = 524288
	const[#WS_TABSTOP] = 65536
	const[#WS_THICKFRAME] = 262144
	const[#WS_TILED] = 0
	const[#WS_VISIBLE] = 268435456
	const[#WS_VSCROLL] = 2097152

	--WS_OVERLAPPEDWINDOW = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX
	const[#WS_OVERLAPPEDWINDOW] = 13565952

	--WS_POPUPWINDOW = WS_POPUP | WS_BORDER | WS_SYSMENU
	const[#WS_POPUPWINDOW] = -2138570752

	--WS_TILEDWINDOW = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX
	const[#WS_TILEDWINDOW] = 13565952

	me.WINDOWSTYLE = const
end

----------------------------------------
--
----------------------------------------
on loadMediaTypeConstants (me)
	const = [:]
	const[#Audio] = "73647561_0000_0010_8000_00AA00389B71"
	const[#FileTransfer] = "D9E47579_930E_4427_ADFC_AD80F290E470"
	const[#Image] = "34A50FD8_8AA5_4386_81FE_A0EFE0488E31"
	const[#Script] = "73636d64_0000_0010_8000_00AA00389B71"
	const[#Text] = "9BBA1EA7_5AB2_4829_BA57_0940209BCF3E"
	const[#Video] = "73646976_0000_0010_8000_00AA00389B71"
	me.MEDIATYPE = const
end
